/*
 * Copied from com.atlassian.confluence.plugins.macros.dashboard.DashboardMacroSupport with some
 * changes (removed unnecessary methods and made 2.10 compatible (loosing some performance
 * improvements (see TODOS)))
 */

package de.communardo.confluence.plugins.displaysubspace;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;

import javax.servlet.http.HttpServletRequest;

import com.atlassian.confluence.core.DateFormatter;
import com.atlassian.confluence.core.FormatSettingsManager;
import com.atlassian.confluence.labels.Label;
import com.atlassian.confluence.labels.LabelManager;
import com.atlassian.confluence.languages.LocaleManager;
import com.atlassian.confluence.setup.settings.Settings;
import com.atlassian.confluence.setup.settings.SettingsManager;
import com.atlassian.confluence.spaces.Space;
import com.atlassian.confluence.spaces.SpaceGroup;
import com.atlassian.confluence.spaces.SpaceGroupManager;
import com.atlassian.confluence.spaces.SpaceManager;
import com.atlassian.confluence.spaces.SpaceType;
import com.atlassian.confluence.spaces.SpacesQuery;
import com.atlassian.confluence.user.AuthenticatedUserThreadLocal;
import com.atlassian.confluence.user.UserAccessor;
import com.atlassian.confluence.user.UserInterfaceState;
import com.atlassian.confluence.util.GeneralUtil;
import com.atlassian.confluence.util.SpaceComparator;
import com.atlassian.confluence.util.SpaceGroupComparator;
import com.atlassian.spring.container.ContainerManager;
import com.atlassian.user.User;
import com.opensymphony.util.TextUtils;
import com.opensymphony.webwork.ServletActionContext;

/**
 * This support class provides common functionality used by the macros that are used on the
 * dashboard.
 */
public class DashboardMacroSupport {
    private LabelManager labelManager;
    private SpaceManager spaceManager;
    private SpaceGroupManager spaceGroupManager;
    private SettingsManager settingsManager;
    private LocaleManager localeManager;

    // These are cached because their getters could get called more than once in a single request
    private List<Space> favSpaces;
    private List<Space> newSpaces;
    private List<Space> teamSpaces;
    private List<Space> groupSpaces;
    private List<String> viewableTeamLabels;
    private List<SpaceGroup> spaceGroups;
    private UserAccessor userAccessor;
    // permittedSpaces is a Set for performance reasons
    private Set<Space> permittedSpaces;

    public List<Space> getFavouriteSpaces() {
        if (favSpaces != null)
            return favSpaces;

        // anonymous users do not have personal labels
        if (getUser() == null) {
            favSpaces = Collections.emptyList();
        } else {
            favSpaces = labelManager.getFavouriteSpaces(getUser().getName());
            favSpaces.retainAll(getPermittedSpaces());
            Collections.sort(favSpaces, new SpaceComparator());
        }
        return favSpaces;
    }

    public DateFormatter getDateFormatter() {
        User user = getUser();
        FormatSettingsManager formatSettingsManager = getFormatSettingsManager();
        return getUserAccessor().getConfluenceUserPreferences(user).getDateFormatter(formatSettingsManager);
    }

    private FormatSettingsManager getFormatSettingsManager() {
        return (FormatSettingsManager) ContainerManager.getComponent("formatSettingsManager");
    }

    public List<Space> getNewSpaces() {
        if (newSpaces != null)
            return newSpaces;

        Calendar cal = getDateFormatter().getCalendar();
        cal.setTime(new Date());
        cal.add(Calendar.DATE, -7);
        newSpaces = spaceManager.getSpacesCreatedAfter(cal.getTime());
        newSpaces.retainAll(getPermittedSpaces());
        return newSpaces;
    }

    public List<Space> getTeamSpaces() {
        if (teamSpaces != null)
            return teamSpaces;

        String selectedTeam = getSelectedTeamLabelName();
        Label label = labelManager.getLabel("team:" + selectedTeam);
        if (label == null) {
            teamSpaces = Collections.emptyList();
        } else {
            teamSpaces = labelManager.getSpacesWithLabel(label);
            teamSpaces.retainAll(getPermittedSpaces());
        }
        return teamSpaces;
    }

    public Set<Space> getPermittedSpaces() {
        if (permittedSpaces == null)
            permittedSpaces = new HashSet<Space>(spaceManager.getAllSpaces(SpacesQuery.newQuery()
                    .forUser(getUser()).build()));

        return permittedSpaces;
    }

    public List<Space> getGroupSpaces() {
        if (groupSpaces == null) {
            String selectedGroup = getSelectedGroupKey();
            SpaceGroup spaceGroup = spaceGroupManager.getSpaceGroup(selectedGroup);
            groupSpaces = spaceManager.getAllSpaces(SpacesQuery.newQuery().forUser(getUser())
                    .inSpaceGroup(spaceGroup).build());
        }
        return groupSpaces;
    }

    protected String getSelectedTeamLabelName() {
        List availableTeams = getViewableTeamLabels();
        return getSelectedTeamLabelName(availableTeams);
    }

    private String getSelectedTeamLabelName(List availableTeams) {
        if (availableTeams == null || availableTeams.size() == 0)
            return "";

        String existingPreference = getUserInterfaceState().getDashboardSpacesSelectedTeam();
        if (TextUtils.stringSet(existingPreference) && availableTeams.contains(existingPreference))
            return existingPreference;
        else
            return (String) availableTeams.get(0);
    }

    protected String getSelectedGroupKey() {
        List availableSpaceGroups = spaceGroupManager.getSpaceGroups();
        if (availableSpaceGroups == null || availableSpaceGroups.size() == 0)
            return "";

        String existingPreference = getUserInterfaceState().getDashboardSpacesSelectedGroup();
        if (!TextUtils.stringSet(existingPreference) || spaceGroupManager.getSpaceGroup(existingPreference) == null)
            return ((SpaceGroup) availableSpaceGroups.get(0)).getKey();
        else
            return existingPreference;
    }

    public List<String> getViewableTeamLabels() {
        if (viewableTeamLabels != null)
            return viewableTeamLabels;

        // TODO: the following code only works with CF 3.4, add it if Macro gets updated
        // List<Label> labels = labelManager.getTeamLabelsForSpaces(getPermittedSpaces());
        // viewableTeamLabels = new ArrayList<String>(labels.size());
        // for (Label label : labels) {
        // viewableTeamLabels.add(label.getName());
        // }

        // this code was added instead
        viewableTeamLabels = new ArrayList<String>();
        for (Space spc : getPermittedSpaces()) {
            for (Label label : (List<Label>) labelManager.getTeamLabelsForSpace(spc.getKey())) {
                viewableTeamLabels.add(label.getName());
            }
        }
        // end of added code

        return viewableTeamLabels;
    }

    // /CLOVER:OFF
    protected boolean isSuperUser() {
        return (GeneralUtil.isSuperUser(getUser()));
    }

    // /CLOVER:ON

    private User getUser() {
        return AuthenticatedUserThreadLocal.getUser();
    }

    // /CLOVER:OFF
    protected UserInterfaceState getUserInterfaceState() {
        return new UserInterfaceState(getUser(), getUserAccessor());
    }

    // /CLOVER:ON

    public String getSpacesSelectedTab() {
        String result;

        if (TextUtils.stringSet(getRequestParameter("spacesSelectedTab"))) {
            getUserInterfaceState().setDashboardTab(getRequestParameter("spacesSelectedTab"));
            result = getRequestParameter("spacesSelectedTab");
        } else // sanitise the selected tab when it is not explicitly set
        {
            if (AuthenticatedUserThreadLocal.getUser() == null) // for anonymous users, when no tab
            // is explicitly set, _always_
            // redirect to "all"
            {
                return "all";
            } else {
                String existingPreference = getUserInterfaceState().getDashboardTab();
                if (TextUtils.stringSet(existingPreference))
                    result = sanitise(existingPreference);
                else
                    result = sanitise("my");
            }
        }

        return result;
    }

    /**
     * four cases to sanitise on
     * <ul>
     * <li>"new" tab when there are no new spaces
     * <li>"my" tab when there are no favourite spaces
     * <li>"team" tab when there are no teams
     * <li>"groups" tab when there are no groups</li>
     * 
     * @param spacesSelectedTab
     *            currently selected tab
     */
    protected String sanitise(String spacesSelectedTab) {
        if ("new".equals(spacesSelectedTab) && getNewSpaces().isEmpty()) {
            return "all";
        } else if ("my".equals(spacesSelectedTab) && getFavouriteSpaces().isEmpty()) {
            return "all";
        } else if ("team".equals(spacesSelectedTab) && getViewableTeamLabels().isEmpty()) {
            return "all";
        } else if ("groups".equals(spacesSelectedTab) && getSpaceGroups().isEmpty()) {
            return "all";
        } else {
            return spacesSelectedTab;
        }
    }

    public String getRequestParameter(String key) {
        HttpServletRequest request = getRequest();
        if (request == null)
            return null;

        return request.getParameter(key);
    }

    /**
     * There are three possible cases here -
     * <ol>
     * <li>You are within an HTTP request thread, in which case the request will be retrieved from a
     * ThreadLocal
     * <li>You are an AJAX call in which case the request will be retrieved from a
     * WebContextFactory.
     * <li>You are not a 'web related' thread (e.g. a Long running task) in which case null will be
     * returned.
     * </ol>
     * 
     * @return The HttpServletRequest associated with the current thread.
     */
    public HttpServletRequest getRequest() {

        HttpServletRequest request = ServletActionContext.getRequest();
        return request;
    }

    private UserAccessor getUserAccessor() {
        if (userAccessor == null)
            userAccessor = (UserAccessor) ContainerManager.getComponent("userAccessor");

        return userAccessor;
    }

    public List<SpaceGroup> getSpaceGroups() {
        if (spaceGroups != null)
            return spaceGroups;

        Settings settings = settingsManager.getGlobalSettings();

        if (settings.isSharedMode()) {
            List<Space> availableSpaces = getAllViewableSpaces();
            Set<SpaceGroup> uniqueGroups = new TreeSet<SpaceGroup>(new SpaceGroupComparator());

            for (Space space : availableSpaces) {
                SpaceGroup group = space.getSpaceGroup();
                if (group != null)
                    uniqueGroups.add(group);
            }
            spaceGroups = new ArrayList<SpaceGroup>(uniqueGroups);
        } else {
            spaceGroups = spaceGroupManager.getSpaceGroups();
            Collections.sort(spaceGroups, new SpaceGroupComparator());
        }

        return spaceGroups;
    }

    private List<Space> getAllViewableSpaces() {
        List<Space> allSpaces = spaceManager.getAllSpaces(SpacesQuery.newQuery().forUser(getUser())
                .withSpaceType(SpaceType.GLOBAL).build());
        Collections.sort(allSpaces, new SpaceComparator());
        return allSpaces;
    }

    public void setLabelManager(LabelManager labelManager) {
        this.labelManager = labelManager;
    }

    public void setSpaceManager(SpaceManager spaceManager) {
        this.spaceManager = spaceManager;
    }

    public void setSpaceGroupManager(SpaceGroupManager spaceGroupManager) {
        this.spaceGroupManager = spaceGroupManager;
    }

    public void setSettingsManager(SettingsManager settingsManager) {
        this.settingsManager = settingsManager;
    }

    public void setLocaleManager(final LocaleManager localeManager) {
        this.localeManager = localeManager;
    }
}
