package de.communardo.confluence.plugins.quicksearch;

import java.util.Collection;
import java.util.List;

import org.apache.commons.lang.StringUtils;

import com.atlassian.confluence.search.actions.SearchSiteAction;
import com.atlassian.confluence.search.service.ContentTypeEnum;
import com.atlassian.confluence.search.service.DateRangeEnum;
import com.atlassian.confluence.search.service.PredefinedSearchBuilder;
import com.atlassian.confluence.spaces.Space;
import com.atlassian.confluence.spaces.SpaceManager;
import com.atlassian.confluence.spaces.SpacesQuery;
import com.atlassian.confluence.user.AuthenticatedUserThreadLocal;
import com.atlassian.confluence.util.GeneralUtil;
import com.atlassian.confluence.velocity.htmlsafe.HtmlSafe;
import com.atlassian.user.User;

import de.communardo.confluence.plugins.configuration.ConfigureHierarchyHelper;
import de.communardo.confluence.plugins.displaysubspace.HierarchyBuilder;
import de.communardo.confluence.plugins.displaysubspace.TreeNode;
import de.communardo.confluence.plugins.util.ComponentManager;

public class AdvancedSearchAction extends SearchSiteAction {

    private boolean includeSubspaces;
    private PredefinedSearchBuilder defaultPredefinedSearchBuilder;
    private Collection<TreeNode> hierarchy;
    private SpaceManager spaceManager;
    private String topLevelSpacekey;

    @Override
    public String execute() throws Exception {

        HierarchyBuilder hierarchyBuilder = ConfigureHierarchyHelper.getHierarchyBuilder();

        User user = AuthenticatedUserThreadLocal.getUser();
        // get the permitted Spaces... possible reason for performance las... use this method
        // carefully
        List<Space> permittedSpaces = ComponentManager.getSpaceManager().getAllSpaces(
                SpacesQuery.newQuery().forUser(user).build());

        hierarchyBuilder.init(permittedSpaces);
        hierarchy = hierarchyBuilder.getHierarchy();
        Space topLevelSpace = spaceManager.getSpace(topLevelSpacekey);

        if (includeSubspaces || topLevelSpace != null) {
            super.setPredefinedSearchBuilder(new AdvancedSearchBuilderDelegator(
                    this.defaultPredefinedSearchBuilder, hierarchyBuilder, topLevelSpace));
        }

        return super.execute();
    }

    @Override
    public void setPredefinedSearchBuilder(PredefinedSearchBuilder predefinedSearchBuilder) {
        this.defaultPredefinedSearchBuilder = predefinedSearchBuilder;
        super.setPredefinedSearchBuilder(predefinedSearchBuilder);
    }

    public void setIncludeSubspaces(boolean includeSubspaces) {
    	this.includeSubspaces = includeSubspaces;
    }
 
    public boolean isIncludeSubspaces() {
        return includeSubspaces;
    }

    public void setHierarchy(Collection<TreeNode> hierarchy) {
        this.hierarchy = hierarchy;
    }

    public Collection<TreeNode> getHierarchy() {
        return hierarchy;
    }

    @Override
    public void setSpaceManager(SpaceManager spaceManager) {
        super.setSpaceManager(spaceManager);
        this.spaceManager = spaceManager;
    }

    public SpaceManager getSpaceManager() {
        return spaceManager;
    }

    public void setTopLevelSpacekey(String topLevelSpacekey) {
        this.topLevelSpacekey = topLevelSpacekey;
    }

    public String getTopLevelSpacekey() {
        return topLevelSpacekey;
    }
    
    @Override
    public CriteriaParameters getCurrentSearch() {
    	return new SubspaceCriteriaParameters(null, getQueryString(), getWhere(), getContentType(), getLastModified(), getContributor(), getContributorUsername(), this.includeSubspaces, this.topLevelSpacekey);
    }
    
    public static class SubspaceCriteriaParameters extends CriteriaParameters {

    	private boolean includeSubspaces;
		private String topLevelSpacekey;

		public SubspaceCriteriaParameters(String labelKey, String queryString, String spaceKey, ContentTypeEnum type,
                DateRangeEnum lastModified, String contributor, String contributorUsername, boolean includeSubspaces, String topLevelSpaceKey) {
    		
    		super( labelKey,  queryString,  spaceKey,  type,
                     lastModified,  contributor,  contributorUsername);
    		this.includeSubspaces = includeSubspaces;
    		this.topLevelSpacekey = topLevelSpaceKey;
    	}
		
		@Override
		@HtmlSafe
		public String getQueryParameters() {
			// TODO Auto-generated method stub
			StringBuffer result =  new StringBuffer();
			result.append(super.getQueryParameters());
			result.append("&").append("includeSubspaces=").append(this.includeSubspaces);

	        if (StringUtils.isNotBlank(this.topLevelSpacekey)){
	        	result.append("&").append("topLevelSpacekey=").append(GeneralUtil.urlEncode(this.topLevelSpacekey));
	        }
			return result.toString();
		}

    }

    
}
