package de.communardo.confluence.plugins.webitem;

import com.atlassian.confluence.plugin.descriptor.web.WebInterfaceContext;
import com.atlassian.confluence.plugin.descriptor.web.conditions.BaseConfluenceCondition;
import com.atlassian.confluence.spaces.Space;
import com.atlassian.confluence.themes.Theme;
import com.atlassian.confluence.themes.ThemeManager;
import com.atlassian.plugin.PluginAccessor;

/**
 * Show the web fragment if the space sidebar is available.
 */
public class SpaceSidebarCondition extends BaseConfluenceCondition {
    private PluginAccessor pluginAccessor;
    private ThemeManager themeManager;
    private static final String KEY_DOCUMETATIONTHEME = "com.atlassian.confluence.plugins.doctheme:documentation";

    private static final String SIDEBAR_PLUGIN_KEY = new String("com.atlassian.confluence.plugins.confluence-space-ia");

    @Override
    protected boolean shouldDisplay(WebInterfaceContext context) {
        Space space = context.getSpace();
        boolean pluginEnabled = pluginAccessor.isPluginEnabled(SIDEBAR_PLUGIN_KEY);
        Theme spaceTheme = themeManager.getSpaceTheme(space.getKey());
        String documentationThemeKey = spaceTheme.getPluginKey() + ":" + spaceTheme.getModuleKey();

        return !pluginEnabled || (pluginEnabled && KEY_DOCUMETATIONTHEME.equals(documentationThemeKey));
    }

    public void setPluginAccessor(PluginAccessor pluginAccessor) {
        this.pluginAccessor = pluginAccessor;
    }

    public void setThemeManager(ThemeManager themeManager) {
        this.themeManager = themeManager;
    }
}
